/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package scrobblerlog

import (
	"os"
	"time"

	"github.com/spf13/viper"
	"go.uploadedlobster.com/scotty/models"
)

type ScrobblerLogBackend struct {
	filePath       string
	includeSkipped bool
}

func (b ScrobblerLogBackend) FromConfig(config *viper.Viper) models.Backend {
	b.filePath = config.GetString("file-path")
	b.includeSkipped = config.GetBool("include-skipped")
	return b
}

func (b ScrobblerLogBackend) ExportListens(oldestTimestamp time.Time) ([]models.Listen, error) {
	file, err := os.Open(b.filePath)
	if err != nil {
		return nil, err
	}

	defer file.Close()

	result, err := Parse(file, b.includeSkipped)
	if err != nil {
		return nil, err
	}

	return result.Listens, nil
}

func (b ScrobblerLogBackend) ImportListens(listens []models.Listen, oldestTimestamp time.Time) (models.ImportResult, error) {
	result := models.ImportResult{
		TotalCount:    len(listens),
		LastTimestamp: oldestTimestamp,
	}

	file, err := os.Create(b.filePath)
	if err != nil {
		return result, err
	}

	defer file.Close()

	log := ScrobblerLog{
		Timezone: "UNKNOWN",
		Client:   "Rockbox unknown $Revision$",
		Listens:  listens,
	}

	lastTimestamp, err := Write(file, &log)

	if err != nil {
		return result, err
	}

	result.LastTimestamp = lastTimestamp
	result.ImportCount = len(listens)
	return result, nil
}
